/*****************************************************************************
* Fichero:			wlandecrypter.c
* Fecha:			23-03-2006
* Autor:			Nilp0inteR (nilp0inter2k6[at]gmail[dot]com)
* Actualizado:			22-11-2006
*
* Descripcion: Generador de diccionario de claves por defecto para los 
* router de Timofonik Zyxel, Xavvy y Comtrend.
*
* Este programa es software libre; puedes redistribuirlo y/o modificarlo
* bajo los terminos de la Licencia Publica General GNU (GPL) publicada
* por la Free Software Foundation; en su version numero 2, o (bajo tu 
* criterio) la ultima version. Mira http://www.fsf.org/copyleft/gpl.txt.
* 
* Este programa se distribuye SIN GARANTIA de ningun tipo.
*
*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//
// Numero de modelos de routers en la base de datos
//
//#define nRouters 10
#define VERSION 0
#define SUBVERSION 5
#define EXTRAVERSION "SuD-1.0"

//
// Tipo de dato Router, con toda la informacion
//
typedef struct Router
{
	char bssid[9];
	char init[8];
	char notas[30];
} tRouter;

//
// Variables globales
//
char hex[16]="0123456789ABCDEF";
int nRouters=0;
tRouter * routers=NULL;

//
// Funciones
//
void readConfig();
void muestraAyuda();
void muestraCabecera();
int buscaBssid(tRouter routers[nRouters], char *bssid);
void imprimeClaves(FILE *fichero, tRouter routers[nRouters], int bssidId, char *keyEnd);

//
// MAIN !!!
//
int main(int argc, char *argv[])
{
	int bssidId, i;
	int validHex=0;
	char endKey[2];
	FILE *fichero;
	
	if(argc<3 || argc>4)
	{
		muestraAyuda();
		return 1;
	}

	muestraCabecera();

	if(strlen(argv[1])!=17)
	{
		fprintf(stderr," [-] Longitud de BSSID invalida\n");
		return 1;
	}
		
	readConfig();

	bssidId=buscaBssid(routers, argv[1]);

	if(bssidId==-1)
	{
		fprintf(stderr, " [-] BSSID no encontrado\n");
		return 1;
	}
	else
	{
		fprintf(stderr, " [+] BSSID: %s\n" 
				" [+] Modelo: %s\n", argv[1], routers[bssidId].notas);

		for(i=0;i<strlen(argv[2]);i++)
			argv[2][i]=toupper(argv[2][i]);

		if(strlen(argv[2])<7 || strlen(argv[2])>9 || strncmp("WLAN_", argv[2], 5) != 0 )
		{
			fprintf(stderr, " [-] ESSID: %s invalido!!\n", argv[2]);
			return 1;
		}
		else
		{
			for(i=0;i<strlen(hex);i++)
			{
				if(argv[2][5]==hex[i])
				{
					validHex++;
					break;
				}
			}
			for(i=0;i<strlen(hex);i++)
			{
				if(argv[2][6]==hex[i])
				{
					validHex++;
					break;
				}
			}

			if(validHex!=2)
			{
				fprintf(stderr, " [-] ESSID: %s invalido!!\n", argv[2]);
				return 1;
			}
			else
			{
				endKey[0]=argv[2][5];
				endKey[1]=argv[2][6];

				fprintf(stderr," [+] ESSID: %s\n", argv[2]);
				
				if(argc>3) // Fichero
				{
					
					fprintf(stderr," [+] Fichero de claves: %s\n", argv[3]);
					fichero=fopen(argv[3], "a+");
					if(fichero!=NULL)
					{
						imprimeClaves(fichero,routers,bssidId,endKey);
						fclose(fichero);
						fprintf(stderr, " [+] Fichero guardado\n");
					}
					else
					{
						fprintf(stderr," [-] Error al abrir el fichero\n");
						return 1;
					}
				}
				else
				{
					fprintf(stderr," [+] Seleccionada salida estandar\n");
					imprimeClaves(stdout,routers,bssidId,endKey);
				}
			}
		}

	}
		
	return 0;
}	

//
// Introduce los datos de los modelos conocidos
//
void datosRouters(FILE * rc)
{
	char buf[256];
	char * p;
	char bufb[9];
	char bufi[8];
	char bufn[30];
	int ret, i;
	int vers=-1;
	int nLinea=0;

	while (!feof(rc)) {
		nLinea++;

		if (!fgets(buf,255,rc))
			return;

		if (strlen(buf)<2)
			continue;

		if (strlen(buf)>254) {
			fprintf(stderr, "readConfig: Linea %d demasiado larga.\n", nLinea);
			exit(-1);
		}

		p=buf+strlen(buf)-1;
		for (p=buf+strlen(buf)-1; p>buf && ( *p==' ' || *p=='\r' || *p=='\n'); p--)
			*p='\0';

		for (p=buf; *p==' '; p++);

		switch(p[0]) {
			case '#':
				continue;
			case 'V':
				if (p[1]!='1' || p[2]!='\0') {
					fprintf(stderr,"readConfig: Version incorrecta.\n");
					exit(-1);
				}
				vers=1;
				continue;
			default:
				if (vers==-1) {
					fprintf(stderr,"readConfig: Falta version.\n");
					exit(-1);
				}
				
				if (strlen(p) < 15) {			
					fprintf(stderr,"readConfig: Linea %d demasiado corta.\n", nLinea);
					exit(-1);
				}

				strncpy(bufb,p,8);
				bufb[8]='\0';
				for (i=0; bufb[i]!='\0'; i++) {
					if (!( \
						(bufb[i]>='0' && bufb[i]<='9') || \
						(bufb[i]>='A' && bufb[i]<='F') || \
						(bufb[i]>='a' && bufb[i]<='f') || \
						(bufb[i]==':') \
					) || bufb[2]!=':' || bufb[5]!=':' ) {
						fprintf(stderr,"readConfig: Error en la mac de la linea %d.\n", nLinea);
						exit(-1);
					}
				}
				p+=8;

				if (p[0]!=' ') {
					fprintf(stderr,"readConfig: Error de sintaxis.\n");
					exit(-1);
				}

				for (; *p==' '; p++);

				strncpy(bufi,p,7);
				bufi[7]='\0';
				for (i=0; bufi[i]!='\0'; i++) {
					if (!( \
						(bufi[i]>='0' && bufi[i]<='9') || \
						(bufi[i]>='A' && bufi[i]<='Z') || \
						(bufi[i]>='a' && bufi[i]<='z') \
					)) {
						fprintf(stderr,"readConfig: Error en la clave de la linea %d.\n", nLinea);
						exit(-1);
					}
				}
				p+=7;

				for (; *p==' '; p++);
				
				strncpy(bufn,p,29);
				bufn[29]='\0';

				//printf("<%s %s %s>\n", bufb, bufi, bufn);
			
				if (!(routers=realloc(routers, sizeof(tRouter) * ++nRouters))) {
					perror("Error en realloc");
					exit(-1);
				}

				strcpy(routers[nRouters-1].bssid, bufb);
				strcpy(routers[nRouters-1].init, bufi);
				strcpy(routers[nRouters-1].notas, bufn);
		}
	}
}

#define HOMERC "/.wlandecrypterrc"
//
// 
//
void readConfig(tRouter routers[nRouters])
{
	FILE *rc;
	const char * home;
	char * homerc;

	rc=fopen("wlandecrypterrc","r");
	if (rc) { datosRouters(rc); return; }

	home=getenv("HOME");
	homerc=malloc( strlen(home) + strlen(HOMERC) + 1 );
	if (!homerc) { perror("Error en malloc."); exit(-1); }
	strcpy(homerc,home);
	strcat(homerc,HOMERC);
	rc=fopen(homerc,"r");
	if (rc) { datosRouters(rc); return; }

	rc=fopen("/etc/wlandecrypterrc","r");
	if (rc) { datosRouters(rc); return; }

	fprintf(stderr,"No encuentro wlandecrypterrc\n"); exit(-1);
}


//
// Muestra la ayuda del programa
//
void muestraCabecera()
{
	fprintf(stderr, "\n  * wlandecrypter %i.%i_%s", VERSION, SUBVERSION, EXTRAVERSION);
	fprintf(stderr, "\n  (c) 2009 SuD http://www.latinsud.com/pub/");
	fprintf(stderr, "\n  (c) 2006 nilp0inter2k6_at_gmail.com - http://www.rusoblanco.com\n");
}
void muestraAyuda()
{
	muestraCabecera();
	fprintf(stderr, "\n  USO: wlandecrypter <bssid> <essid> [output file]\n\n");
}


//
// Busca el bssid en la estructura y devuelve el identificador o
//  -1 si no existe
//
int buscaBssid(tRouter routers[nRouters], char *bssid)
{
	int i;
	
	for(i=0;i<strlen(bssid);i++)
		bssid[i]=toupper(bssid[i]);

	for(i=0;i<nRouters;i++)
	{
		if(strncmp ( routers[i].bssid, bssid, 8 ) == 0)
			return i;
	}
	
	return -1;
}

//
// Imprime las claves en un fichero 
//
void imprimeClaves(FILE *fichero, tRouter routers[nRouters], int bssidId, char *keyEnd)
{
	int i,j,k,l;
	
	for(i=0;i<16;i++)
	        for(j=0;j<16;j++)
	                for(k=0;k<16;k++)
	                        for(l=0;l<16;l++)
	                        {
					fprintf(fichero, "%s%c%c%c%c%c%c\n",routers[bssidId].init,hex[i],hex[j],hex[k],hex[l],keyEnd[0],keyEnd[1]);
	                        }
}

